import _assertThisInitialized from "@babel/runtime/helpers/assertThisInitialized";
import _inheritsLoose from "@babel/runtime/helpers/inheritsLoose";
import _defineProperty from "@babel/runtime/helpers/defineProperty";
import React from "react";
import { findDOMNode } from "react-dom";
import classNames from "classnames";
import ListItem from "./ListItem";
import ListItemGroup from "./ListItemGroup";
import scrollIntoView from "dom-helpers/scrollTo";
export var version = "2.0.1";
var KEY_UP = 38;
var KEY_DOWN = 40;
var KEY_ENTER = 13;
// function isIE() {
// 	const userAgent = navigator.userAgent;
// 	const isIE = userAgent.indexOf("compatible") > -1 && userAgent.indexOf("MSIE") > -1;
// 	const isIE11 = userAgent.indexOf("Trident") > -1 && userAgent.indexOf("rv:11.0") > -1;
// 	return isIE || isIE11;
// }
export function dataProcessor(props) {
  var data = props.data,
      valueField = props.valueField,
      childrenField = props.childrenField,
      labelField = props.labelField,
      disabledField = props.disabledField;
  var items = [];
  var itemList = [];
  var itemsMap = {};

  function walk(dataset, pChildren) {
    dataset.forEach(function (data) {
      var item = {
        value: data[valueField],
        label: data[labelField],
        disabled: data[disabledField],
        data: data,
        ref: React.createRef()
      };
      itemList.push(item);
      pChildren.push(item);
      itemsMap[data[valueField]] = item;

      if (data[childrenField] && Array.isArray(data[childrenField])) {
        walk(data[childrenField], item.children = []);
      }
    });
  }

  walk(data, items);
  return {
    items: items,
    itemList: itemList,
    itemsMap: itemsMap
  };
}
export var ListBox = /*#__PURE__*/function (_React$Component) {
  _inheritsLoose(ListBox, _React$Component);

  ListBox.getDerivedStateFromProps = function getDerivedStateFromProps(nextProps, state) {
    if (nextProps === state.prevProps) {
      return null;
    }

    return Object.assign(Object.assign({}, nextProps.computedState || dataProcessor(nextProps)), {}, {
      value: nextProps.value === undefined ? state.value : Array.isArray(nextProps.value) ? nextProps.value : [nextProps.value],
      prevProps: nextProps
    });
  };

  function ListBox(props, context) {
    var _this;

    _this = _React$Component.call(this, props, context) || this;

    _defineProperty(_assertThisInitialized(_this), "onKeyDown", function (e) {
      var activeItem = _this.state.activeItem;
      var keyCode = e.keyCode;

      if (keyCode === KEY_DOWN || keyCode === KEY_UP) {
        e.preventDefault();

        _this.setState({
          activeItem: _this.getNextActiveItem(keyCode)
        }, function () {
          _this.scrollActiveItemIntoView();
        });
      } else if (activeItem && keyCode === KEY_ENTER) {
        _this.toggleSelectItem(activeItem);
      }
    });

    _defineProperty(_assertThisInitialized(_this), "handleMouseLeave", function (e) {
      var onMouseLeave = _this.props.onMouseLeave;

      _this.setState({
        activeItem: null
      });

      onMouseLeave === null || onMouseLeave === void 0 ? void 0 : onMouseLeave(e);
    });

    _defineProperty(_assertThisInitialized(_this), "handleItemMouseEnter", function (cValue) {
      _this.setState({
        activeItem: _this.getItemByValue(cValue)
      });
    });

    _defineProperty(_assertThisInitialized(_this), "handleItemMouseLeave", function (cValue) {
      _this.setState({
        activeItem: null
      });
    });

    _defineProperty(_assertThisInitialized(_this), "handleItemClick", function (cValue, e) {
      var item = _this.getItemByValue(cValue);

      if (!item) return;
      scrollIntoView(e.currentTarget);
      if (item.disabled) return;

      _this.toggleSelectItem(item);
    });

    _defineProperty(_assertThisInitialized(_this), "getItemProps", function (data) {
      var getItemProps = _this.props.getItemProps;
      return getItemProps ? getItemProps(data) : {};
    });

    _this.state = {
      items: [],
      itemList: [],
      itemsMap: {},
      value: props.defaultValue === undefined ? [] : Array.isArray(props.defaultValue) ? props.defaultValue : [props.defaultValue]
    };
    return _this;
  } // protected refHandlers = {};


  var _proto = ListBox.prototype;

  _proto.focus = function focus() {
    findDOMNode(this).focus();
  };

  _proto.blur = function blur() {
    findDOMNode(this).blur();
  };

  _proto.getItemByValue = function getItemByValue(value) {
    return this.state.itemsMap[value] || null;
  };

  _proto.fireSelect = function fireSelect(item) {
    var onSelect = this.props.onSelect;
    onSelect === null || onSelect === void 0 ? void 0 : onSelect(item.value, item);
  };

  _proto.fireDeselect = function fireDeselect(item) {
    var onDeselect = this.props.onDeselect;
    onDeselect === null || onDeselect === void 0 ? void 0 : onDeselect(item.value, item);
  };

  _proto.setValue = function setValue(newValue) {
    var _this2 = this;

    var _this$props = this.props,
        multiple = _this$props.multiple,
        onChange = _this$props.onChange;

    if (this.props.value === undefined) {
      this.setState({
        value: newValue
      });
    }

    if (multiple) {
      var items = newValue.map(function (value) {
        var _this2$getItemByValue;

        return (_this2$getItemByValue = _this2.getItemByValue(value)) === null || _this2$getItemByValue === void 0 ? void 0 : _this2$getItemByValue.data;
      });
      onChange === null || onChange === void 0 ? void 0 : onChange(newValue, items);
    } else {
      if (!newValue.length) return;

      var _item = this.getItemByValue(newValue[0]);

      onChange === null || onChange === void 0 ? void 0 : onChange(newValue[0], _item === null || _item === void 0 ? void 0 : _item.data);
    }
  };

  _proto.scrollActiveItemIntoView = function scrollActiveItemIntoView() {
    var activeItem = this.state.activeItem;

    if (activeItem && activeItem.ref.current) {
      scrollIntoView(activeItem.ref.current.node);
    }
  };

  _proto.getActiveIndex = function getActiveIndex() {
    var items = this.state.itemList.filter(function (item) {
      return !item.children;
    });
    var currentActive = this.state.activeItem || this.getItemByValue(this.state.value[0]) || items[0];
    if (!currentActive) return -1;

    for (var i = 0; i < items.length; i++) {
      if (currentActive.value === items[i].value) {
        return i;
      }
    }

    return -1;
  };

  _proto.getNextActiveItem = function getNextActiveItem(code) {
    var items = this.state.itemList.filter(function (item) {
      return !item.children;
    });
    var activeItem = null;
    if (!items.length) return activeItem;
    var activeIndex = this.getActiveIndex();

    while (--items.length) {
      if (code === KEY_DOWN) {
        activeIndex++;
      } else if (code === KEY_UP) {
        activeIndex--;

        if (activeIndex < 0) {
          activeIndex = items.length - 1;
        }
      }

      activeIndex = activeIndex % items.length;
      activeItem = items[activeIndex];
      if (activeItem && activeItem.disabled) continue;
      break;
    }

    return activeItem;
  };

  _proto.toggleSelectItem = function toggleSelectItem(item) {
    var _this$props2 = this.props,
        multiple = _this$props2.multiple,
        readOnly = _this$props2.readOnly,
        disabled = _this$props2.disabled;
    var value = this.state.value;
    var idx = value.indexOf(item.value);

    if (readOnly || disabled) {
      return;
    }

    if (multiple) {
      if (idx === -1) {
        this.setValue([].concat(value, [item.value]));
      } else {
        var newValue = [].concat(value);
        newValue.splice(idx, 1);
        this.setValue(newValue);
      }

      if (idx !== -1) {
        this.fireSelect(item);
      } else {
        this.fireDeselect(item);
      }
    } else if (idx === -1) {
      this.setValue([item.value]);
    }
  };

  _proto.renderListItems = function renderListItems(items) {
    var _this3 = this;

    var _this$props3 = this.props,
        prefixCls = _this$props3.prefixCls,
        renderItem = _this$props3.renderItem,
        renderGroupTitle = _this$props3.renderGroupTitle,
        getGroupTitleProps = _this$props3.getGroupTitleProps;
    var _this$state = this.state,
        activeItem = _this$state.activeItem,
        value = _this$state.value;
    return items.map(function (item) {
      var itemPrefixCls = prefixCls + "-item";
      var groupPrefixCls = prefixCls + "-group";
      return item.children ? /*#__PURE__*/React.createElement(ListItemGroup, {
        getGroupTitleProps: getGroupTitleProps,
        prefixCls: groupPrefixCls,
        item: item,
        renderGroupTitle: renderGroupTitle
      }, _this3.renderListItems(item.children)) : /*#__PURE__*/React.createElement(ListItem, {
        key: item.value,
        ref: item.ref,
        prefixCls: itemPrefixCls,
        data: item.data,
        value: item.value,
        disabled: item.disabled,
        getItemProps: _this3.getItemProps,
        selected: value.indexOf(item.value) !== -1,
        active: (activeItem === null || activeItem === void 0 ? void 0 : activeItem.value) === item.value,
        onClick: _this3.handleItemClick,
        onMouseEnter: _this3.handleItemMouseEnter,
        onMouseLeave: _this3.handleItemMouseLeave
      }, renderItem ? renderItem(item.data, item) : item.label);
    });
  };

  _proto.renderList = function renderList() {
    var emptyLabel = this.props.emptyLabel;
    var items = this.state.items;

    if (!items.length) {
      return emptyLabel;
    }

    return this.renderListItems(items);
  } //修正IE对flex支持不够完善
  // resizeListBoxBodyHeight() {
  // 	const dom = this.getListView();
  // 	const header = this.getListViewHeader();
  // 	const footer = this.getListViewFooter();
  // 	const body = this.getListViewBody();
  // 	body.style.maxHeight = "";
  // 	const hasScroll = dom.scrollHeight > dom.clientHeight;
  // 	const headerHeight = header ? header.offsetHeight : 0;
  // 	const footerHeight = footer ? footer.offsetHeight : 0;
  // 	if (hasScroll) {
  // 		body.style.maxHeight = dom.clientHeight - headerHeight - footerHeight + "px";
  // 	}
  // }
  ;

  _proto.componentDidMount = function componentDidMount() {
    var _this$props4 = this.props,
        autoFocus = _this$props4.autoFocus,
        fixListBodyHeightOnIE = _this$props4.fixListBodyHeightOnIE;
    var value = this.state.value; // if (isIE() && fixListBodyHeightOnIE) {
    // 	this.resizeListBoxBodyHeight();
    // }

    if (autoFocus) {
      this.focus();
    }

    if (value.length) {
      var _item2 = this.getItemByValue(value[0]);

      if (_item2 && _item2.ref.current) {
        scrollIntoView(_item2.ref.current.node);
      }
    }
  };

  _proto.renderBody = function renderBody() {
    var _this$props5 = this.props,
        BodyWrapperComponent = _this$props5.bodyWrapperComponent,
        prefixCls = _this$props5.prefixCls,
        bodyStyle = _this$props5.bodyStyle;
    return /*#__PURE__*/React.createElement(BodyWrapperComponent, {
      className: prefixCls + "-body",
      style: bodyStyle
    }, this.renderList());
  };

  _proto.render = function render() {
    var _classNames;

    var _this$props6 = this.props,
        className = _this$props6.className,
        prefixCls = _this$props6.prefixCls,
        tabIndex = _this$props6.tabIndex,
        disabled = _this$props6.disabled,
        readOnly = _this$props6.readOnly,
        onFocus = _this$props6.onFocus,
        onBlur = _this$props6.onBlur,
        style = _this$props6.style,
        WrapperComponent = _this$props6.wrapperComponent,
        renderer = _this$props6.renderer;
    var classes = classNames((_classNames = {}, _classNames["" + prefixCls] = true, _classNames[className] = className, _classNames[prefixCls + "-disabled"] = disabled, _classNames[prefixCls + "-readonly"] = readOnly, _classNames));
    var listBody = this.renderBody();
    return /*#__PURE__*/React.createElement(WrapperComponent, {
      tabIndex: tabIndex,
      className: classes,
      style: style,
      onKeyDown: this.onKeyDown,
      onFocus: onFocus,
      onBlur: onBlur
    }, renderer ? renderer(listBody) : listBody);
  };

  return ListBox;
}(React.Component);

_defineProperty(ListBox, "defaultProps", {
  prefixCls: "rw-listbox",
  valueField: "value",
  labelField: "label",
  disabledField: "disabled",
  childrenField: "children",
  tabIndex: 0,
  emptyLabel: "Not Found",
  fixListBodyHeightOnIE: true,
  data: [],
  wrapperComponent: "div",
  bodyWrapperComponent: "div"
});

export default ListBox;