import React from "react";
import { findDOMNode } from "react-dom";
import classNames from "classnames";
import ListItem from "./ListItem";
import ListItemGroup from "./ListItemGroup";
import scrollIntoView from "dom-helpers/scrollTo";
export const version = "%VERSION%";
const KEY_UP = 38;
const KEY_DOWN = 40;
const KEY_ENTER = 13;
// function isIE() {
// 	const userAgent = navigator.userAgent;
// 	const isIE = userAgent.indexOf("compatible") > -1 && userAgent.indexOf("MSIE") > -1;
// 	const isIE11 = userAgent.indexOf("Trident") > -1 && userAgent.indexOf("rv:11.0") > -1;
// 	return isIE || isIE11;
// }
export function dataProcessor(props) {
    const { data, valueField, childrenField, labelField, disabledField } = props;
    const items = [];
    const itemList = [];
    const itemsMap = {};
    function walk(dataset, pChildren) {
        dataset.forEach((data) => {
            const item = {
                value: data[valueField],
                label: data[labelField],
                disabled: data[disabledField],
                data,
                ref: React.createRef(),
            };
            itemList.push(item);
            pChildren.push(item);
            itemsMap[data[valueField]] = item;
            if (data[childrenField] && Array.isArray(data[childrenField])) {
                walk(data[childrenField], (item.children = []));
            }
        });
    }
    walk(data, items);
    return {
        items,
        itemList,
        itemsMap,
    };
}
let ListBox = /** @class */ (() => {
    class ListBox extends React.Component {
        constructor(props, context) {
            super(props, context);
            this.onKeyDown = (e) => {
                const { activeItem } = this.state;
                const keyCode = e.keyCode;
                if (keyCode === KEY_DOWN || keyCode === KEY_UP) {
                    e.preventDefault();
                    this.setState({
                        activeItem: this.getNextActiveItem(keyCode),
                    }, () => {
                        this.scrollActiveItemIntoView();
                    });
                }
                else if (activeItem && keyCode === KEY_ENTER) {
                    this.toggleSelectItem(activeItem);
                }
            };
            this.handleMouseLeave = (e) => {
                const { onMouseLeave } = this.props;
                this.setState({
                    activeItem: null,
                });
                onMouseLeave?.(e);
            };
            this.handleItemMouseEnter = (cValue) => {
                this.setState({
                    activeItem: this.getItemByValue(cValue),
                });
            };
            this.handleItemMouseLeave = (cValue) => {
                this.setState({
                    activeItem: null,
                });
            };
            this.handleItemClick = (cValue, e) => {
                const item = this.getItemByValue(cValue);
                if (!item)
                    return;
                scrollIntoView(e.currentTarget);
                if (item.disabled)
                    return;
                this.toggleSelectItem(item);
            };
            this.getItemProps = (data) => {
                const { getItemProps } = this.props;
                return getItemProps ? getItemProps(data) : {};
            };
            this.state = {
                items: [],
                itemList: [],
                itemsMap: {},
                value: props.defaultValue === undefined
                    ? []
                    : Array.isArray(props.defaultValue)
                        ? props.defaultValue
                        : [props.defaultValue],
            };
        }
        static getDerivedStateFromProps(nextProps, state) {
            if (nextProps === state.prevProps) {
                return null;
            }
            return {
                ...(nextProps.computedState || dataProcessor(nextProps)),
                value: nextProps.value === undefined
                    ? state.value
                    : Array.isArray(nextProps.value)
                        ? nextProps.value
                        : [nextProps.value],
                prevProps: nextProps,
            };
        }
        // protected refHandlers = {};
        focus() {
            findDOMNode(this).focus();
        }
        blur() {
            findDOMNode(this).blur();
        }
        getItemByValue(value) {
            return this.state.itemsMap[value] || null;
        }
        fireSelect(item) {
            const { onSelect } = this.props;
            onSelect?.(item.value, item);
        }
        fireDeselect(item) {
            const { onDeselect } = this.props;
            onDeselect?.(item.value, item);
        }
        setValue(newValue) {
            const { multiple, onChange } = this.props;
            if (this.props.value === undefined) {
                this.setState({
                    value: newValue,
                });
            }
            if (multiple) {
                const items = newValue.map((value) => this.getItemByValue(value)?.data);
                onChange?.(newValue, items);
            }
            else {
                if (!newValue.length)
                    return;
                const item = this.getItemByValue(newValue[0]);
                onChange?.(newValue[0], item?.data);
            }
        }
        scrollActiveItemIntoView() {
            const { activeItem } = this.state;
            if (activeItem && activeItem.ref.current) {
                scrollIntoView(activeItem.ref.current.node);
            }
        }
        getActiveIndex() {
            const items = this.state.itemList.filter((item) => !item.children);
            let currentActive = this.state.activeItem || this.getItemByValue(this.state.value[0]) || items[0];
            if (!currentActive)
                return -1;
            for (let i = 0; i < items.length; i++) {
                if (currentActive.value === items[i].value) {
                    return i;
                }
            }
            return -1;
        }
        getNextActiveItem(code) {
            const items = this.state.itemList.filter((item) => !item.children);
            let activeItem = null;
            if (!items.length)
                return activeItem;
            let activeIndex = this.getActiveIndex();
            while (--items.length) {
                if (code === KEY_DOWN) {
                    activeIndex++;
                }
                else if (code === KEY_UP) {
                    activeIndex--;
                    if (activeIndex < 0) {
                        activeIndex = items.length - 1;
                    }
                }
                activeIndex = activeIndex % items.length;
                activeItem = items[activeIndex];
                if (activeItem && activeItem.disabled)
                    continue;
                break;
            }
            return activeItem;
        }
        toggleSelectItem(item) {
            const { multiple, readOnly, disabled } = this.props;
            const { value } = this.state;
            const idx = value.indexOf(item.value);
            if (readOnly || disabled) {
                return;
            }
            if (multiple) {
                if (idx === -1) {
                    this.setValue([...value, item.value]);
                }
                else {
                    const newValue = [...value];
                    newValue.splice(idx, 1);
                    this.setValue(newValue);
                }
                if (idx !== -1) {
                    this.fireSelect(item);
                }
                else {
                    this.fireDeselect(item);
                }
            }
            else if (idx === -1) {
                this.setValue([item.value]);
            }
        }
        renderListItems(items) {
            const { prefixCls, renderItem, renderGroupTitle, getGroupTitleProps } = this.props;
            const { activeItem, value } = this.state;
            return items.map((item) => {
                const itemPrefixCls = `${prefixCls}-item`;
                const groupPrefixCls = `${prefixCls}-group`;
                return item.children ? (React.createElement(ListItemGroup, { getGroupTitleProps: getGroupTitleProps, prefixCls: groupPrefixCls, item: item, renderGroupTitle: renderGroupTitle }, this.renderListItems(item.children))) : (React.createElement(ListItem, { key: item.value, ref: item.ref, prefixCls: itemPrefixCls, data: item.data, value: item.value, disabled: item.disabled, getItemProps: this.getItemProps, selected: value.indexOf(item.value) !== -1, active: activeItem?.value === item.value, onClick: this.handleItemClick, onMouseEnter: this.handleItemMouseEnter, onMouseLeave: this.handleItemMouseLeave }, renderItem ? renderItem(item.data, item) : item.label));
            });
        }
        renderList() {
            const { emptyLabel } = this.props;
            const { items } = this.state;
            if (!items.length) {
                return emptyLabel;
            }
            return this.renderListItems(items);
        }
        //修正IE对flex支持不够完善
        // resizeListBoxBodyHeight() {
        // 	const dom = this.getListView();
        // 	const header = this.getListViewHeader();
        // 	const footer = this.getListViewFooter();
        // 	const body = this.getListViewBody();
        // 	body.style.maxHeight = "";
        // 	const hasScroll = dom.scrollHeight > dom.clientHeight;
        // 	const headerHeight = header ? header.offsetHeight : 0;
        // 	const footerHeight = footer ? footer.offsetHeight : 0;
        // 	if (hasScroll) {
        // 		body.style.maxHeight = dom.clientHeight - headerHeight - footerHeight + "px";
        // 	}
        // }
        componentDidMount() {
            const { autoFocus, fixListBodyHeightOnIE } = this.props;
            const { value } = this.state;
            // if (isIE() && fixListBodyHeightOnIE) {
            // 	this.resizeListBoxBodyHeight();
            // }
            if (autoFocus) {
                this.focus();
            }
            if (value.length) {
                const item = this.getItemByValue(value[0]);
                if (item && item.ref.current) {
                    scrollIntoView(item.ref.current.node);
                }
            }
        }
        renderBody() {
            const { bodyWrapperComponent: BodyWrapperComponent, prefixCls, bodyStyle } = this.props;
            return (React.createElement(BodyWrapperComponent, { className: `${prefixCls}-body`, style: bodyStyle }, this.renderList()));
        }
        render() {
            const { className, prefixCls, tabIndex, disabled, readOnly, onFocus, onBlur, style, wrapperComponent: WrapperComponent, renderer, } = this.props;
            const classes = classNames({
                [`${prefixCls}`]: true,
                [className]: className,
                [`${prefixCls}-disabled`]: disabled,
                [`${prefixCls}-readonly`]: readOnly,
            });
            const listBody = this.renderBody();
            return (React.createElement(WrapperComponent, { tabIndex: tabIndex, className: classes, style: style, onKeyDown: this.onKeyDown, onFocus: onFocus, onBlur: onBlur }, renderer ? renderer(listBody) : listBody));
        }
    }
    ListBox.defaultProps = {
        prefixCls: "rw-listbox",
        valueField: "value",
        labelField: "label",
        disabledField: "disabled",
        childrenField: "children",
        tabIndex: 0,
        emptyLabel: "Not Found",
        fixListBodyHeightOnIE: true,
        data: [],
        wrapperComponent: "div",
        bodyWrapperComponent: "div",
    };
    return ListBox;
})();
export { ListBox };
export default ListBox;
